#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 18 11:42:01 2019
Some general rules of testing:
    # A testing unit should focus on one tiny bit of functionality and prove it
      correct.
    # Each test unit must be fully independent.
      Each test must be able to run alone, and also within the test suite,
      regardless of the order that they are called.
    # Try hard to make tests that run fast.
    # Learn your tools and learn how to run a single test or a test case.
    # Always run the full test suite before a coding session, and run it again
      after.
    # It is a good idea to implement a hook that runs all tests before pushing
      code to a shared repository.
    # If you are in the middle of a development session and have to interrupt
      your work, it is a good idea to write a broken unit test about what you
      want to develop next.
    # The first step when you are debugging your code is to write a new test
      pinpointing the bug. While it is not always possible to do,
      those bug catching tests are among the most valuable pieces of code in
      your project.
    # Use long and descriptive names for testing functions.
    # Another use of the testing code is as an introduction to new developers.

@author: shlomi
"""
import unittest


def f(n):
    """pre_factor for scipy.special.eval_hermite function for comparison with
    pymaws _eval_hermite_polynomial function.
    input : n integer and scaler"""
    import numpy as np
    return (2**n * np.math.factorial(n) * np.sqrt(np.pi))**-0.5


class Test_pymaws(unittest.TestCase):
    def test_hermite(self):
        """Testing _eval_hermite_polynomial in pymaws vs.
            scipy.special.eval_hermite with prefactor correction:
            1)Random sample from x, and choose 1 to 10 n's.
            2)Run _eval_hermite_polynomial(x, n).
            3)Run eval_hermite(n, x) from spipy.special multiplied by f(n).
            f(n)=(2**n * np.math.factorial(n) * np.sqrt(np.pi))**-0.5.
            4)The error is the vector norm of the difference between the two
              results.
            5)The error for each n should be lower than the threshold (1e-6).
            The error ~ n because of the factorial in f(n)."""
        import numpy as np
        from pymaws import _eval_hermite_polynomial
        from scipy.special import eval_hermite
        print('Testing pymaws._eval_hermite_polynomial: ')
        thresh = 1e-6
        ns = np.arange(1, 11)
        x = 20.0 * np.random.rand(100) - 10.0
        print('Using ' + str(len(ns)) + ' n`s from ' + str(min(ns)) + ' to ' +
              str(max(ns)) + ':')
        print('Using ' + str(len(x)) + ' random x`s from ' +
              str(round(min(x), 2)) + ' to ' + str(round(max(x), 2)) + ':')
        for n in ns:
            # calculate with pymaws function:
            totest = _eval_hermite_polynomial(x, n)
            # calculate with scipy function:
            test = f(n) * eval_hermite(n, x)
            # error is the norm of the x vector for each n:
            error = np.linalg.norm(test - totest)
            self.assertLessEqual(error, abs(thresh),
                                 'Error should be' +
                                 ' smaller than ' + str(thresh))
        print('_eval_hermite_polynomial test complete with threshold of ' +
              str(thresh))
        print('')

    def test_omega(self):
        """Testing _eval_omega in pymaws vs. numpy.roots:
           1)Randomly sample 25 k's and n's from 1-50.
           2)For each wave-type(Rossby, EIG, WIG) run _eval_omega(k, n)
             for Earth's parametes.
           3)Run np.roots with the coeffs of the cubic eq.#2 in:
             https://doi.org/10.5194/gmd-2018-260
             (Note: we scale k = k/A only for np.roots run.
           4)The error is the sum of the difference of steps 2,3.
           5)The error should be lower than the threshold of 1e-18.
           6)The sum of all three wave-types in step 2 should be lower than the
            threshold of 1e-18."""
        import numpy as np
        from pymaws import _eval_omega
        from pymaws import Earth
        from pymaws import _unpack_parameters
        print('Testing pymaws._eval_omega: ')
        # unpack dictionary into vars:
        OMEGA = _unpack_parameters(Earth, 'angular_frequency')
        G = _unpack_parameters(Earth, 'gravitational_acceleration')
        A = _unpack_parameters(Earth, 'mean_radius')
        H0 = _unpack_parameters(Earth, 'layer_mean_depth')
        ns = np.random.randint(1, 51, 25)
        ks = np.random.randint(1, 51, 25)
        print('Using ' + str(len(ns)) + ' random n`s from ' + str(min(ns)) +
              ' to ' + str(max(ns)) + ':')
        print('Using ' + str(len(ks)) + ' random k`s from ' + str(min(ks)) +
              ' to ' + str(max(ks)) + ':')
        test = {'Rossby': 0, 'WIG': 0, 'EIG': 0}
        totest = test.copy()
        thresh = 1e-18
        for k in ks:
            for n in ns:
                coeffs = [-1, 0, (G * H0) * (k / A)**2 + 2 * OMEGA *
                          (2 * n + 1) * (G * H0)**0.5 / A, (2 * OMEGA * G *
                          H0 * k / A) / A]
                roots = np.roots(coeffs)
                test.update(Rossby=-min(abs(roots)), WIG=min(roots),
                            EIG=max(roots))
                totest.update(_eval_omega(k, n, Earth))
                error = {k: test[k] - totest[k] for k in test.keys()}
                totest_sum = sum(test.values())
                self.assertLessEqual(sum([abs(x) for x in error.values()]),
                                     thresh, 'Error should be' +
                                     ' smaller than ' + str(thresh))
                self.assertLessEqual(totest_sum, thresh,
                                     'Sum of all 3 roots ' +
                                     'should be smaller than ' + str(thresh))
        print('_eval_omega test complete with threshold of ' + str(thresh))
        print('')

    def test_v(self):
        """Testing _eval_meridional_velocity in pymaws using
            np.trapz, basically testing for orthonormallity of the meridional
            velocity functions:
           1)Compute the Lamb parameter for Earth.
           2)Assaign x to be latitude range from -80 to 80 in radians.
           3)Sample an amplitude from 1e-15 to 1e15 uniformly.
           4)Run _eval_meridional_velocity with n=10, x ,Lamb, amp
           5)Run _eval_meridional_velocity with n=5, x ,Lamb, amp
           6)The error is an integral of step 4 times step 5 times a scaling
            constant.
           7)The term that should equal one is the integral of step 4 times
             step 4 (or step 5 times step 5) times the same scaling factor
             from step 6."""
        from pymaws import _eval_meridional_velocity
        from pymaws import Earth
        from pymaws import _unpack_parameters
        import numpy as np
        import random
        print('Testing _eval_meridional_velocity: ')
        # unpack dictionary into vars:
        OMEGA = _unpack_parameters(Earth, 'angular_frequency')
        G = _unpack_parameters(Earth, 'gravitational_acceleration')
        A = _unpack_parameters(Earth, 'mean_radius')
        H0 = _unpack_parameters(Earth, 'layer_mean_depth')
        Lamb = (2. * OMEGA * A)**2 / (G * H0)
        thresh = 1e-15
        lats = np.deg2rad(np.linspace(-80., 80., 100))
        print('Using ' + str(len(lats)) + ' lats from ' +
              str(round(min(lats), 2)) + ' to ' +
              str(round(max(lats), 2)) + ':')
        amp = random.uniform(np.log(1e-15), np.log(1e15))
        amp = np.exp(amp)
        print('Using amplitude: {0:.2g}'.format(amp))
        v_10 = _eval_meridional_velocity(lats, Lamb, n=10, amp=amp)
        v_5 = _eval_meridional_velocity(lats, Lamb, n=5, amp=amp)
        error = np.trapz(v_10 * v_5, lats) * Lamb**0.25 / (amp**2)
        one = np.trapz(v_10 * v_10, lats) * Lamb**0.25 / (amp**2)
        self.assertLessEqual(error, thresh, 'Error should be' +
                             ' smaller than ' + str(thresh))
        self.assertAlmostEqual(one, 1.0, 7)
        print('_eval_meridional_velocity test complete with threshold of ' +
              str(thresh))
        print('')

    def test_eval_field_amplitudes(self):
        """Testing _eval_field_amplitudes in pymaws:
           1)Sample random n's and k's from 1-15 and from 1-50 respectively.
           2)Sample 10 latitudes from -80 to 80 in radians.
           3)Sample 1 radnom amplitude from 1e-15 to 1e15 uniformly.
           4)For each wave_type and n and k, compute the omega using
            _eval_omega and v_hat, u_hat, phi_hat
            the error term is (-1j * omega * u_hat - 2 * OMEGA * lats * v_hat +
                             1j * k / (A) * phi_hat) / amp
           5)The final error term is sum(abs(np.real(error from step 4)))
        """
        from pymaws import _eval_field_amplitudes
        from pymaws import _eval_omega
        from pymaws import _unpack_parameters
        from pymaws import Earth
        import random
        import numpy as np
        print('Testing pymaws._eval_field_amplitudes: ')
        OMEGA = _unpack_parameters(Earth, 'angular_frequency')
        A = _unpack_parameters(Earth, 'mean_radius')
        # ns = np.arange(1, 11)
        ns = np.random.randint(1, 16, 10)
        ks = np.random.randint(1, 51, 25)
        print('Using ' + str(len(ns)) + ' random n`s from ' + str(min(ns)) +
              ' to ' + str(max(ns)) + ':')
        print('Using ' + str(len(ks)) + ' random k`s from ' + str(min(ks)) +
              ' to ' + str(max(ks)) + ':')
        lats = np.deg2rad(80.0 * np.random.rand(10) - 40.0)
        print('Using ' + str(len(lats)) + ' random lats from ' +
              str(round(min(lats), 2)) + ' to ' + str(round(max(lats), 2)) +
              ':')
        waves = ['Rossby', 'EIG', 'WIG']
        thresh = 1e-15
        for wave in waves:
            print('Testing ' + wave + ' wave_type:')
            amp = random.uniform(np.log(1e-15), np.log(1e15))
            amp = np.exp(amp)
            # print("Sammy ate {0:.3f} percent of a pizza!".format(75.765367))
            print('Using amplitude: {0:.2g}'.format(amp))
            for k in ks:
                for n in ns:
                    omega = _eval_omega(k, n, Earth)[wave]
                    v_hat = _eval_field_amplitudes(lats, k, n, amp, 'v', wave,
                                                   Earth)
                    phi_hat = _eval_field_amplitudes(lats, k, n, amp, 'phi',
                                                     wave, Earth)
                    u_hat = _eval_field_amplitudes(lats, k, n, amp, 'u', wave,
                                                   Earth)
                    error = (-1j * omega * u_hat - 2 * OMEGA * lats * v_hat +
                             1j * k / (A) * phi_hat) / amp
                    summed_error = sum(abs(np.real(error)))
                    self.assertLessEqual(summed_error,
                                         thresh, 'Error should be' +
                                         ' smaller than ' + str(thresh))
            print(' Test for ' + wave +
                  ' done with threshold of ' + str(thresh))
        print('_eval_meridional_velocity test for all wave types done' +
              ' with threshold of ' + str(thresh))
        print('')

    def test_eval_field(self):
        """Testing eval_field in pymaws:
           1)Sample 10 latitudes from -80 to 80 in radians.
           2)Sample 1 radnom amplitude from 1e-15 to 1e15 uniformly.
           3)pick lon1, lon2, time1 and time2 to be 1, 2, 1, 1.01 respectivly.
           4)Run eval_field for v, u(two times) and phi(two lons).
           5)The error term is ((u2 - u1) / (time2 - time1) - 2 * OMEGA * lats
             * v1 + 1.0 / (A) * (phi2 - phi1) / (lon2 - lon1)) / amp
           6)The summed error term is sum(abs(error from step 5))
            """
        from pymaws import eval_field
        from pymaws import Earth
        from pymaws import _unpack_parameters
        import numpy as np
        import random
        print('Testing pymaws.eval_field: ')
        thresh = 1e-5
        OMEGA = _unpack_parameters(Earth, 'angular_frequency')
        A = _unpack_parameters(Earth, 'mean_radius')
        lats = np.deg2rad(80.0 * np.random.rand(10) - 40.0)
        print('Using ' + str(len(lats)) + ' random lats from ' +
              str(round(min(lats), 2)) + ' to ' + str(round(max(lats), 2)) +
              ':')
        lon1 = np.deg2rad(1.0)
        lon2 = np.deg2rad(2.0)
        time1 = 1.0
        time2 = 1.01
        amp = random.uniform(np.log(1e-15), np.log(1e15))
        amp = np.exp(amp)
        print('Using amplitude: {0:.2g}'.format(amp))
        nj = lats.shape[0]
        v1 = np.zeros((nj))
        u1 = np.zeros((nj))
        u2 = np.zeros((nj))
        phi1 = np.zeros((nj))
        phi2 = np.zeros((nj))
        for j in range(nj):
            v1[j] = eval_field(lats[j], lon1, time1, field='v', amp=amp)
            u1[j] = eval_field(lats[j], lon1, time1, field='u', amp=amp)
            u2[j] = eval_field(lats[j], lon1, time2, field='u', amp=amp)
            phi1[j] = eval_field(lats[j], lon1, time1, field='phi', amp=amp)
            phi2[j] = eval_field(lats[j], lon2, time1, field='phi', amp=amp)
        error = ((u2 - u1) / (time2 - time1) - 2 * OMEGA * lats *
                 v1 + 1.0 / (A) * (phi2 - phi1) / (lon2 - lon1)) / amp
        summed_error = sum(abs(error))
        self.assertLessEqual(summed_error,
                             thresh, 'Error should be' +
                             ' smaller than ' + str(thresh))
        print('')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar 14 17:02:46 2019

@author: shlomi
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pymaws",
    version="0.1.0",
    author="Ofer Shamir",
    author_email="ofer.shamir@mail.huji.ac.il",
    description="Matsuno Analytical Wave Solution implemented in Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ofershamir/matsuno",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar 13 11:03:39 2019
@author: shlomi
"""
Earth = {
        'angular_frequency': 7.29212e-5,
        'gravitational_acceleration': 9.80616,
        'mean_radius': 6371220.,
        'layer_mean_depth': 30.
        }


def _unpack_parameters(parameters, key):
    """Unpacks the values from a dictionary containing various parameters for
    pymaws

    Parameters
    ----------
    parameters : dict
        planetary parameters dict with keys:
            angular_frequency: float, (rad/sec)
            gravitational_acceleration: float, (m/sec^2)
            mean_radius: float, (m)
            layer_mean_depth: float, (m)
    key : String
        key with the one of the names of the above keys.
    Returns
    -------
    value : Float, scalar
            one of the above values
    """
    if not isinstance(key, str):
        raise TypeError(str(key) + ' should be string...')
    if not isinstance(parameters, dict):
        raise TypeError('parameters should be dictionary...')
    if key not in parameters:
        raise KeyError(str(key) + ' not in parameters!')
    value = parameters[key]
    return value


def _eval_omega(k, n, parameters=Earth):
    """
    Evaluates the wave-frequencies for a given wave-number and wave-mode.
    For further details see Eqs. (2)-(5) in  the text

    Parameters
    ----------
    k : Integer, scalar
        spherical wave-number (dimensionless). must be >= 1.
    n : Integer, scaler
        wave-mode (dimensionless). must be >=1.
    parameters: dict
        planetary parameters dict with keys:
            angular_frequency: float, (rad/sec)
            gravitational_acceleration: float, (m/sec^2)
            mean_radius: float, (m)
            layer_mean_depth: float, (m)
        Defualt: Earth's parameters defined above

    Returns
    -------
    omega : Float, dict
            wave frequency in rad/sec for each wave-type(Rossby, EIG, WIG)

    Notes
    -----
    This function supports k>=1 and n>=1 inputs only.
    Special treatments are required for k=0 and n=-1,0/-.

    """
    import numpy as np
    # make sure k and n are scalers:
    if not np.isscalar(n):
        raise TypeError('n must be scalar')
    if not np.isscalar(k):
        raise TypeError('k must be scalar')
    # make sure input is integer:
    if not isinstance(k, (int, np.integer)):
        raise TypeError('k should be integer, i.e., ' + str(int(k)))
    if not isinstance(n, (int, np.integer)):
        raise TypeError('n should be integer, i.e., ' + str(int(n)))
    # check for k>=1 and n>=1
    if k < 1:
        raise ValueError('pymaws supports only k>=1 for now...')
    if n < 1:
        raise ValueError('pymaws supports only n>=1 for now...')
    # unpack dictionary into vars:
    OMEGA = _unpack_parameters(parameters, 'angular_frequency')
    G = _unpack_parameters(parameters, 'gravitational_acceleration')
    A = _unpack_parameters(parameters, 'mean_radius')
    H0 = _unpack_parameters(parameters, 'layer_mean_depth')
    # evaluate Eq. (4) the text
    omegaj = np.zeros((1, 3))
    delta0 = 3. * (G * H0 * (k / A)**2 + 2. * OMEGA *
                   (G * H0)**0.5 / A * (2 * n + 1))
    delta4 = -54. * OMEGA * G * H0 * k / A**2

    for j in range(1, 4):
        deltaj = (delta4**2 - 4. * delta0**3 + 0. * 1j)**0.5
        deltaj = (0.5 * (delta4 + deltaj))**(1. / 3.)
        deltaj = deltaj * np.exp(2. * np.pi * 1j * j / 3.)
        # evaluate Eq. (3) the text
        omegaj[0, j - 1] = np.real(-1. / 3. * (deltaj + delta0 / deltaj))
    # put all wave-types in dict:
    # (Eq. (5) in the text)
    omega = {'Rossby': -np.min(np.abs(omegaj)),
             'WIG': np.min(omegaj),
             'EIG': np.max(omegaj)}
    return omega


def _eval_hermite_polynomial(x, n):
    """
    Evaluates the normalized Hermite polynomial of degree n at point/s x
    using the three-term recurrence relation. For further details see Eq. (7)
    in the text.

    Parameters
    ----------
    x : Float, array_like or scalar
        list or array of points where the evalution takes place.
    n : Integer, scaler
        degree of the Hermite polynomial.

    Returns
    -------
    H_n : Float, array_like or scalar
            Evaluation of the normalized Hermite polynomial.

    Notes
    -----

    """
    import numpy as np
    # make sure n is integer and scaler:
    if not np.isscalar(n):
        raise TypeError('n must be scalar')
    if not isinstance(n, (int, np.integer)):
        raise TypeError('n should be integer, i.e., ' + str(int(n)))
    # make sure x is an array(or scalar):
    x = np.asarray(x)
    # main evaluation:
    if n < 0:
        H_n = np.zeros(x.shape)
    elif n == 0:
        H_n = np.ones(x.shape) / np.pi**0.25
    elif n == 1:
        H_n = (4.0 / np.pi)**0.25 * x
    elif n >= 2:
        H_n = ((2.0 / n)**0.5 * x * _eval_hermite_polynomial(x, n - 1) -
               ((n - 1) / n)**0.5 * _eval_hermite_polynomial(x, n - 2))
    return H_n


def _eval_meridional_velocity(lat, Lamb, n=1, amp=1e-5):
    """
    Evaluates the meridional velocity amplitude at a given latitude point and
    a given wave-amplitude. See Eq.(6a) in the text.

    Parameters
    ----------
    lat : Float, array_like or scalar
          latitude(radians)
    Lamb: Float, scalar
          Lamb's parameter. ~ 2935 for Earth's parameters
    n : Integer, scaler
        wave-mode (dimensionless)
        Default : 1
    amp : Float, scalar
          wave amplitude(m/sec)
          Default : 1e-5

    Returns
    -------
    vel : Float, array_like or scalar
          Evaluation of the meridional velocity.

    Notes
    -----
    This function supports n>=1 inputs only.
    Special treatments are required for n=-1,0/-.

    """
    import numpy as np
    if not np.isscalar(amp):
        raise TypeError('amp must be scalar')
    # re-scale latitude
    y = Lamb**0.25 * lat

    # Gaussian envelope
    ex = np.exp(-0.5 * y**2)

    vel = amp * ex * _eval_hermite_polynomial(y, n)

    return vel


def _eval_field_amplitudes(lat, k=5, n=1, amp=1e-5, field='v',
                           wave_type='Rossby', parameters=Earth):
    """
    Evaluates the latitude dependent amplitudes at a given latitude point.

    Parameters
    ----------
    lat : Float, array_like or scalar
          latitude(radians)
    k : Integer, scalar
        spherical wave-number (dimensionless)
        Default : 5
    n : Integer, scaler
        wave-mode (dimensionless)
        Default : 1
    amp : Float, scalar
          wave amplitude(m/sec)
          Default : 1e-5
    field : str
            pick 'phi' for geopotential height,
            'u' for zonal velocity and 'v' for meridional velocity
            Defualt : 'v'
    wave_type: str
        choose Rossby waves or WIG waves or EIG waves.
        Defualt: Rossby
    parameters: dict
        planetary parameters dict with keys:
            angular_frequency: float, (rad/sec)
            gravitational_acceleration: float, (m/sec^2)
            mean_radius: float, (m)
            layer_mean_depth: float, (m)
        Defualt: Earth's parameters defined above
    Returns
    -------
    Either u_hat(m/sec), v_hat(m/sec) or p_hat(m^2/sec^2) : Float, array_like
    or scalar Evaluation of the amplitudes for the zonal velocity,
    or meridional velocity or the geopotential height respectivly.

    Notes
    -----
    This function supports k>=1 and n>=1 inputs only.
    Special treatments are required for k=0 and n=-1,0/-.

    """
    if not isinstance(wave_type, str):
        raise TypeError(str(wave_type) + ' should be string...')
    # unpack dictionary into vars:
    OMEGA = _unpack_parameters(parameters, 'angular_frequency')
    G = _unpack_parameters(parameters, 'gravitational_acceleration')
    A = _unpack_parameters(parameters, 'mean_radius')
    H0 = _unpack_parameters(parameters, 'layer_mean_depth')
    # Lamb's parameter:
    Lamb = (2. * OMEGA * A)**2 / (G * H0)
    # evaluate wave frequency:
    all_omegas = _eval_omega(k, n, parameters)
    # check for validity of wave_type:
    if wave_type not in all_omegas:
        raise KeyError(wave_type + ' should be Rossby, EIG or WIG...')
    omega = all_omegas[wave_type]
    # evaluate the meridional velocity amp first:
    v_hat = _eval_meridional_velocity(lat, Lamb, n, amp)
    # evaluate functions for u and phi:
    v_hat_plus_1 = _eval_meridional_velocity(lat, Lamb, n + 1, amp)
    v_hat_minus_1 = _eval_meridional_velocity(lat, Lamb, n - 1, amp)
    # Eq. (6a) in the text
    if field == 'v':
        return v_hat
    # Eq. (6b) in the text
    elif field == 'u':
        u_hat = (- ((n + 1) / 2.0)**0.5 * (omega / (G * H0)**0.5 + k / A) *
                 v_hat_plus_1 - ((n) / 2.0)**0.5 * (omega / (G * H0)**0.5 -
                                                    k / A) * v_hat_minus_1)
        # pre-factors
        u_hat = G * H0 * Lamb**0.25 / \
            (1j * A * (omega**2 - G * H0 * (k / A)**2)) * u_hat
        return u_hat
    # Eq. (6c) in the text
    elif field == 'phi':
        p_hat = (- ((n + 1) / 2.0)**0.5 * (omega + (G * H0)**0.5 * k / A) *
                 v_hat_plus_1 + ((n) / 2.0)**0.5 * (omega - (G * H0)**0.5 *
                                                    k / A) * v_hat_minus_1)
        p_hat = G * H0 * Lamb**0.25 / \
            (1j * A * (omega**2 - G * H0 * (k / A)**2)) * p_hat
        return p_hat
    else:
        raise KeyError('field must be u, v or phi')


def eval_field(lat, lon, time, k=5, n=1, amp=1e-5, field='phi',
               wave_type='Rossby', parameters=Earth):
    """
    Evaluates the analytic solutions of either the zonal or meridional velocity
    or the geopotential height on at given latitude, longitude and time.

    Parameters
    ----------
    lat : Float, scalar
          latitude(radians)
    lon : Float, scalar
          longitude(radians)
    time : Float, scaler
           time(sec), =0 if one wants initial conditions.
    k : Integer, scalar
        spherical wave-number (dimensionless)
        Default : 5
    n : Integer, scaler
        wave-mode (dimensionless)
        Default : 1
    amp : Float, scalar
          wave amplitude(m/sec)
          Defualt : 1e-5
    field : str
            pick 'phi' for geopotential height,
            'u' for zonal velocity and 'v' for meridional velocity
            Defualt : 'phi'
    wave_type: str
        choose Rossby waves or WIG waves or EIG waves.
        Defualt: Rossby
    parameters: Float, dict
        planetary parameters dict with keys:
            angular_frequency: float, (rad/sec)
            gravitational_acceleration: float, (m/sec^2)
            mean_radius: float, (m)
            layer_mean_depth: float, (m)
        Defualt: Earth's parameters defined above
    Returns
    -------
    f : Float, scalar
        Evaluation of the the zonal velocity(m/sec),
        or meridional velocity(m/sec) or the geopotential height(m^2/sec^2)
        respectivly.

    Notes
    -----
    This function supports k>=1 and n>=1 inputs only.
    Special treatments are required for k=0 and n=-1,0/-.

    """
    import numpy as np

    # make sure lat, lon and time are scalars
    if not np.isscalar(lat):
        raise TypeError('lat must be scalar')
    if not np.isscalar(lon):
        raise TypeError('lon must be scalar')
    if not np.isscalar(time):
        raise TypeError('time must be scalar')

    # frequency
    all_omegas = _eval_omega(k, n, parameters)
    if wave_type not in all_omegas:
        raise KeyError(wave_type + ' should be Rossby, EIG or WIG...')
    omega = all_omegas[wave_type]

    # latitude-dependent amplitudes
    if field == 'phi':
        f_hat = _eval_field_amplitudes(lat, k, n, amp, 'phi', wave_type,
                                       parameters)
    elif field == 'u':
        f_hat = _eval_field_amplitudes(lat, k, n, amp, 'u', wave_type,
                                       parameters)
    elif field == 'v':
        f_hat = _eval_field_amplitudes(lat, k, n, amp, 'v', wave_type,
                                       parameters)

    # adding time and longitude dependence
    f = np.real(np.exp(1j * (k * lon - omega * time)) * f_hat)

    return f
